//
// client.cpp
// ~~~~~~~~~~
//
// Copyright (c) 2003-2013 Christopher M. Kohlhoff (chris at kohlhoff dot com)
//
// Distributed under the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
//

#include <iostream>
#include <boost/array.hpp>
#include <boost/asio.hpp>

using namespace std;

using boost::asio::ip::tcp;

int main(int argc, char* argv[])
{
  if (argc != 3) {
    std::cerr << "Usage: client <host> <port>" << std::endl;
    return 1;
  }
  
  try
  {
    boost::system::error_code error;

    boost::asio::io_service service;

    tcp::resolver resolver(service);
    tcp::resolver::query query(argv[1], argv[2]);
    tcp::resolver::iterator iterator = resolver.resolve(query);
    
    tcp::socket socket(service);
    tcp::resolver::iterator end;
    while (iterator != end) {
      tcp::endpoint endpoint = *iterator;
      
      cout << "tentative de connexion avec " << endpoint << "...";
      cout.flush();
      socket.connect(endpoint, error);
      if (!error) {
	cerr << "réussie !" << endl;
	break;
      }
      cerr << "echouée !" << endl;
      socket.close();
      iterator++;
    }
    if (iterator==end) {
      cerr << "Impossible de se connecter" << endl;
      throw boost::system::system_error(error);
    }
    
    
    
    
    // d'abord on lit le message de bienvenu (terminé par \r\n)
    boost::asio::streambuf sbuf;
    size_t len = boost::asio::read_until(socket, sbuf, "\r\n", error);
    if (error == boost::asio::error::eof){
      cout << "fermeture de la socket par le serveur" << endl;
      return 0;
    }
    if (error) {
      cerr << "Il y a un problème de lecture sur la socket" << error.message() << endl;
      throw boost::system::system_error(error);
    }

    // transformation du buffer de lecture en string
    istream is(&sbuf);
    sbuf.commit(len);
    //    string mess(sbuf.data(), len);
    string mess;
    getline(is, mess);
    
    cout << "Le serveur a envoyé " << mess << endl;
    // cout << "toto" << endl;
    // cout << "Le serveur à ecrit '" << mess << "'" << endl;

    while (1) {
      string text;
      getline (cin, text);
      cout << "Message à envoyer '" << text << "'" << endl;
      if (text == "bye") {
	break;
      }
      socket.write_some(boost::asio::buffer(text.c_str(), text.size()),error);
      if (error) {
	cerr << "impossible d'écrire dans la socket" << endl;
	throw boost::system::system_error(error);
      }      
    }
  }
  catch (std::exception& e)
  {
    std::cerr << e.what() << std::endl;
  }

  return 0;
}
